/*
 * Copyright (c) 2003 Jens Mueller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/**
 	Data class for representing set-requests

 	@author Jens Mueller
 */

import java.io.PrintWriter;
import java.util.Vector;
import java.util.Iterator;

public class GatewaySetRequest
{
	public Vector _contextRequestVector;
 
	public GatewaySetRequest()
	{
		_contextRequestVector = new Vector();
	}
	
	
	/**
	 	prepares and initates SNMP-communication for performing this request
	 	@throws RequestExecutionException
	 */
	public void execute() throws RequestExecutionException
	{
		String errorstr = "";
		for( Iterator iterator = _contextRequestVector.iterator(); iterator.hasNext(); )
		{
			try
			{
				( ( ContextSetRequest )iterator.next() ).execute();
			}
			catch( RequestExecutionException e )
			{
				errorstr = e.getMessage();
			}
		}
		if( errorstr != "" )
			throw new RequestExecutionException( errorstr );
	}
	
	
	/**
	 	writes a textual representation of this request to <code>writer</code> for debugging
	 	@param writer
	 */
	public void dump( PrintWriter writer )
	{
		for( Iterator iterator = _contextRequestVector.iterator(); iterator.hasNext(); )
			( ( ContextSetRequest )iterator.next() ).dump( writer );	
	}
}
